/*
 * IM2VID
 * -------------------------------------------
 * MIGUEL TORRES TORRITI (c) 2007 v. 1.0
 * -------------------------------------------
 *
 * To run type:
 *  im2vid [input image basefilename=f] [options] [output video=video.avi]
 *
 * Description: 
 *  Creates a video from a sequence of images.  By default this function 
 *  loads files f000.jpg, f001.jpg, ..., f010.jpg and generates a video named video.avi
 *  at 15 fps.
 *
 *  Note:
 *  The option to select video codecs for BSD/Linux is not supported in the current version,
 *  and needs recompiling the source files with the codec parameter appropriatly set in the 
 *  CvVideoWriter structure instead of using -1.
 *
 * Options:
 *  -h   Prints this help.
 *  -nv  Turns verbose off.
 *  -np  Do not wait for user to press ESC when done.
 *  -nd  Do not display images.
 *  -no  Do not save the output.
 *  -e   Filename extension [=jpg]
 *  -sq initial_value final_value  Applies im2vid to a sequence of images
 *                                 starting at filename<initial_value>.ext ending in filename<initial_value>.ext
 *            By default the sequence range is assumed to be between 0 and 10 
 *            with zero-padding = 3, i.e. 000, 001, ..., 010.
 *  -zp num  Pad filename suffix with num zeros.
 *  -fps num   Sets the frame rate (in frames per second).
 *             By default fps is assumed to be 15 fps.
 *
 * Author: Miguel Torres Torriti (c) 2007.12.15, v. 1.
 *
 */
#include "cv.h"       // use " " instead of < > to get list of class/struct members
#include "highgui.h"  // use " " instead of < > to get list of class/struct members
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>

struct cmdlinestruct {
	char basefilename[256];		// Frame base filename
	char fileextension[250];	// Frame file extension
	char filenameformat[200];	// "%s%0<n>d.%s" (filename)(zero-padding number)(extension)
	int iv;						// Frame initial value
	int fv;						// Frame final value
	double fps;					// Video writer parameters
	char filename_out[512];		// Output video filename
	bool output_flag;
	bool display_flag;
	bool verbose_flag;
	bool pause_flag;
};

int initcmdlinestruct(cmdlinestruct* cmdlineparam)
{
	sprintf((*cmdlineparam).basefilename,"%s","f");			// Frame base filename
	sprintf((*cmdlineparam).fileextension,"%s","jpg");		// Frame file extension
	sprintf((*cmdlineparam).filenameformat,"%%s%%0%dd.%%s",3);	// Builds file suffix format string "%s%0<n>d.%s"
																// where <n> is the number for zero padding.
	(*cmdlineparam).iv=0;									// Frame initial value
	(*cmdlineparam).fv=10;									// Frame final value
	(*cmdlineparam).fps=15.0;								// Video writer parameters
	sprintf((*cmdlineparam).filename_out,"%s","video.avi");	// Output video filename
	(*cmdlineparam).output_flag=1;
	(*cmdlineparam).display_flag=1;
	(*cmdlineparam).verbose_flag=1;
	(*cmdlineparam).pause_flag=1;
	return 0;
}

int parsecmdline(int argc, char** argv, cmdlinestruct* cmdlineparam)
{
	// Parse command line options
	for (int i=1; i<argc; i++)
	{	
		if (strcspn( argv[i], "-" )==0 && strstr(argv[i],"-nv")!=NULL)
		{
			(*cmdlineparam).verbose_flag = 0;
			break;
		}
	}
	if ((*cmdlineparam).verbose_flag) printf("Verbose is on.\n");

	for (int i=1; i<argc; i++)
	{	
		if (strcspn( argv[i], "-" )==0)
		{
			if (strstr(argv[i],"-nv")!=NULL)
			{
				(*cmdlineparam).verbose_flag = 0;
			}
			else if (strstr(argv[i],"-np")!=NULL)
			{
				(*cmdlineparam).pause_flag = 0;
			}
			else if (strstr(argv[i],"-nd")!=NULL)
			{
				if ((*cmdlineparam).verbose_flag) printf("No display.\n");
				(*cmdlineparam).display_flag = 0;
			}
			else if (strstr(argv[i],"-no")!=NULL)
			{
				if ((*cmdlineparam).verbose_flag) printf("No output.\n");
				(*cmdlineparam).output_flag = 0;
			}
			else if (strstr(argv[i],"-h")!=NULL)
			{
				printf(
					"im2vid [input image basefilename=f] [options] [output video=video.avi]\n"
					"\n"
					"Description:\n"
					"\tCreates a video from a sequence of images.  By default this function\n" 
					"\tloads files f000.jpg, f001.jpg, ..., f010.jpg and generates a video\n"
					"\tnamed video.avi at 15 fps.\n"
					"\n"
					"\tNote:\n"
					"\tThe option to select video codecs for BSD/Linux is not supported in\n"
					"\tthe current version, and needs recompiling the source files with the\n"
					"\tcodec parameter appropriatly set in the CvVideoWriter structure\n"
					"\tinstead of using -1.\n"
					"\n"
					"Options:\n"
					"\t-h   Prints this help.\n"
					"\t-nv  Turns verbose off.\n"
					"\t-np  Do not wait for user to press ESC when done.\n"
					"\t-nd  Do not display images.\n"
					"\t-no  Do not save the output.\n"
					"\t-e   Filename extension [=jpg]\n"
					"\t-sq initial_value final_value\n"
					"\t     Applies im2vid to a sequence of images\n"
					"\t     starting at filename<initial_value>.ext and\n"
					"\t     ending in filename<initial_value>.ext\n"
					"\t     By default the sequence range is assumed to be between 0 and 10\n" 
					"\t     with zero-padding = 3, i.e. 000, 001, ..., 010.\n"
					"\t-zp num  Pad filename suffix with num zeros.\n"
					"\t-fps num Sets the frame rate (in frames per second).\n"
					"\t         By default fps is assumed to be 15 fps.\n"
					"\n"
					"Author: Miguel Torres Torriti (c) 2007.12.15, v. 1.\n"
				);
				return -2;
			}
			else if (strstr(argv[i],"-e")!=NULL)
			{
				sprintf((*cmdlineparam).fileextension,"%s",argv[i+1]);
				i++;
			}
			else if (strstr(argv[i],"-sq")!=NULL)
			{
				(*cmdlineparam).iv=atoi(argv[i+1]);
				(*cmdlineparam).fv=atoi(argv[i+2]);
				i=i+2;
				if ((*cmdlineparam).verbose_flag) printf("Sequence range [v0, v1] = [%d, %d]\n",(*cmdlineparam).iv,(*cmdlineparam).fv);
			}
			else if (strstr(argv[i],"-zp")!=NULL)
			{
				sprintf((*cmdlineparam).filenameformat,"%%s%%0%dd.%%s",atoi(argv[i+1]));	// Builds file suffix format string "%s%0<n>d"
																							// where <n> is the number for zero padding.
				i++;
			}
			else if (strstr(argv[i],"-fps")!=NULL)
			{
				(*cmdlineparam).fps = atof(argv[i+1]);			
				i++;
			}
			else
			{
				if ((*cmdlineparam).verbose_flag) printf("Invalid option detected: %s\n", argv[i]);
			}
		}
		else if (i==1)
		{
			// Generate the output filename tokenizing the input filename
			sprintf((*cmdlineparam).basefilename, "%s", argv[i]);
			if ((*cmdlineparam).verbose_flag) printf("Input base filename : %s\n", (*cmdlineparam).basefilename);
		}
		else
		{
			sprintf((*cmdlineparam).filename_out, "%s", argv[i]);
			if ((*cmdlineparam).verbose_flag) printf("New output filename: %s\n", (*cmdlineparam).filename_out);
		}
	}

	return 0;
}


int main( int argc, char** argv )
{
	IplImage* frame = 0;	       // Frame
	char filename[512];            // Frame filename
	int fnum=0;		               // Frame current number
	struct cmdlinestruct cmdlineparam;	   // Command line parameters structure

	initcmdlinestruct( &cmdlineparam );		

	// Parse the command line parameters 
	if (parsecmdline(argc, argv, &cmdlineparam)==-2)
		return 0;

	// Print welcome message and the OpenCV version
	if (cmdlineparam.verbose_flag)
	{
		printf ("vid2im: Conversion from video to image sequence using OpenCV version %s (%d.%d.%d)\n",
	    CV_VERSION, CV_MAJOR_VERSION, CV_MINOR_VERSION, CV_SUBMINOR_VERSION);

		printf( "Hot keys: \n"
			    "\tESC - quit the program\n"
				"Type im2vid -h for additional help.\n" );
	}

	fnum = cmdlineparam.iv;
	// Create filename for first image and extract its dimensions to set the size of subsequent frames.
	sprintf(filename, cmdlineparam.filenameformat, cmdlineparam.basefilename, fnum, cmdlineparam.fileextension);
	frame = cvLoadImage(filename, CV_LOAD_IMAGE_COLOR);
	if( !frame )
		return -1;

	cvNamedWindow( "im2vid", 0 );
	//cvMoveWindow( "im2vid", 0, 0 );
	//cvShowImage( "im2vid", frame );


	// Set up video writer parameters
	CvVideoWriter* cvVideoWriterStruct = cvCreateVideoWriter(cmdlineparam.filename_out, -1, cmdlineparam.fps, cvSize((*frame).width,(*frame).height), 1 );

	for(int fnum = cmdlineparam.iv; fnum <= cmdlineparam.fv; fnum++)
    {
		frame = 0;

		double t = (double)cvGetTickCount();
	
		sprintf(filename, cmdlineparam.filenameformat, cmdlineparam.basefilename, fnum, cmdlineparam.fileextension);
		frame = cvLoadImage(filename, CV_LOAD_IMAGE_COLOR);
		
		cvWriteFrame( cvVideoWriterStruct, frame );

        if( !frame )
			break;

		cvShowImage( "im2vid", frame );

		int c = cvWaitKey(10);
        if( (char)c == 27 )
            break;

		t = (double)cvGetTickCount() - t;
		printf( "File: %s | processing time = %.2f ms/frame - fps = %.2f\n", 
			filename,
			t/((double)cvGetTickFrequency()*1000.),
			((double)cvGetTickFrequency()*1.e6)/t);
	}

	// Save output
	if (cmdlineparam.output_flag)
	{
		printf("Output video saved to file: %s\n", cmdlineparam.filename_out);
	}
	if (cmdlineparam.verbose_flag)
	{
		printf("Done!\n");
		if (cmdlineparam.pause_flag)
		{
			printf("Press ESC to quit.\n");
			while( getch()!= 0x1B); // Wait till ESC is pressed
		}
	}

	cvReleaseVideoWriter( &cvVideoWriterStruct );
    cvDestroyWindow("im2vid");

	return 0;
}
